@@ -19,7 +19,7 @@
  * including the two.
  *
  * You must obey the GNU Affero General Public License in all respects
- * for all of the code used other than OpenSSL.  
+ * for all of the code used other than OpenSSL.
  */
 
 
@@ -64,6 +64,12 @@
 #define ERROR	System_error
 
 
+/**
+ * The standard debug routine
+ */
+#define DEBUG   System_debug
+
+
 /* ------------------------------------------------------------ Exceptions */
 
 
