@@ -105,7 +105,7 @@ struct _usergroups {
 /* Search the env list and return the pointer to the name (in the list)
  if found, otherwise NULL */
 static inline char *_findEnv(T C, const char *name) {
-        for (list_t p = C->env->head; p; p = p->next) {
+        for (_list_t p = C->env->head; p; p = p->next) {
                 if ((strncmp(p->e, name, strlen(name)) == 0))
                         if (((char*)p->e)[strlen(name)] == '=') // Ensure that p->e is not just a sub-string
                                 return p->e;
@@ -166,7 +166,7 @@ static inline char **_env(T C) {
 /* Create stdio pipes for communication between parent and child process */
 static void _createPipes(Process_T P) {
         if (pipe(P->stdin_pipe) < 0 || pipe(P->stdout_pipe) < 0 || pipe(P->stderr_pipe) < 0) {
-                ERROR("Command pipe(2): Bad file descriptors -- %s", System_getLastError());
+                ERROR("Command pipe(2): Bad file descriptors -- %s", System_lastError());
         }
 }
 
@@ -176,19 +176,19 @@ static void _setupChildPipes(Process_T P) {
         close(P->stdin_pipe[1]);   // close write end
         if (P->stdin_pipe[0] != STDIN_FILENO) {
                 if (dup2(P->stdin_pipe[0],  STDIN_FILENO) != STDIN_FILENO)
-                        ERROR("Command: dup2(stdin) -- %s\n", System_getLastError());
+                        ERROR("Command: dup2(stdin) -- %s\n", System_lastError());
                 close(P->stdin_pipe[0]);
         }
         close(P->stdout_pipe[0]);  // close read end
         if (P->stdout_pipe[1] != STDOUT_FILENO) {
                 if (dup2(P->stdout_pipe[1], STDOUT_FILENO) != STDOUT_FILENO)
-                        ERROR("Command: dup2(stdout) -- %s\n", System_getLastError());
+                        ERROR("Command: dup2(stdout) -- %s\n", System_lastError());
                 close(P->stdout_pipe[1]);
         }
         close(P->stderr_pipe[0]);  // close read end
         if (P->stderr_pipe[1] != STDERR_FILENO) {
                 if (dup2(P->stderr_pipe[1], STDERR_FILENO) != STDERR_FILENO)
-                        ERROR("Command: dup2(stderr) -- %s\n", System_getLastError());
+                        ERROR("Command: dup2(stderr) -- %s\n", System_lastError());
                 close(P->stderr_pipe[1]);
         }
 }
@@ -562,11 +562,8 @@ List_T Command_getCommand(T C) {
 }
 
 
-/* The Execute function. Note that we use vfork() rather than fork. Vfork has
- a special semantic in that the child process runs in the parent address space
- until exec is called in the child. The child also run first and suspend the
- parent process until exec or exit is called
- 
+/* The Execute function.
+
  Note: For possible better error reporting, set exec_error to an enum of possible
  errors and exit with that error. Return Process_T regardless and introduce a
  char *Process_getError() which uses waitpid() to get the error status from child exit.
@@ -581,20 +578,26 @@ Process_T Command_execute(T C) {
         if (C->uid) {
                 struct passwd *user = getpwuid(C->uid);
                 if (!user) {
-                        ERROR("Command: uid %d not found on the system -- %s\n", C->uid, System_getLastError());
+                        ERROR("Command: uid %d not found on the system -- %s\n", C->uid, System_lastError());
                         return NULL;
                 }
                 Command_setEnv(C, "HOME", user->pw_dir);
-                if (getgrouplist(user->pw_name, C->gid, (int *)ug.groups, &ug.ngroups) == -1) {
-                        ERROR("Command: getgrouplist for uid %d -- %s\n", C->uid, System_getLastError());
+                if (getgrouplist(user->pw_name, C->gid,
+#ifdef __APPLE__
+                                 (int *)ug.groups,
+#else
+                                 ug.groups,
+#endif
+                                 &ug.ngroups) == -1) {
+                        ERROR("Command: getgrouplist for uid %d -- %s\n", C->uid, System_lastError());
                         return NULL;
                 }
         }
         Process_T P = _Process_new();
-        int descriptors = System_getDescriptorsGuarded();
+        int descriptors = System_descriptors(1024);
         _createPipes(P);
-        if ((P->pid = vfork()) < 0) {
-                ERROR("Command: fork failed -- %s\n", System_getLastError());
+        if ((P->pid = fork()) < 0) {
+                ERROR("Command: fork failed -- %s\n", System_lastError());
                 Process_free(&P);
                 return NULL;
         } else if (P->pid == 0) {
