@@ -54,8 +54,9 @@
 #include "SMTP.h"
 
 // libmonit
-#include "system/Time.h"
 #include "util/Str.h"
+#include "system/Time.h"
+#include "system/Random.h"
 #include "exceptions/IOException.h"
 
 
@@ -100,8 +101,8 @@ static char *_getFQDNhostname(char host[256]) {
 
 
 static void _substitute(Mail_T m, Event_T e) {
-        ASSERT(m);
-        ASSERT(e);
+        assert(m);
+        assert(e);
 
         if (Str_sub(m->from->name, "$HOST"))
                 Util_replaceString(&m->from->name, "$HOST", _getFQDNhostname(m->host));
@@ -145,8 +146,8 @@ static void _escape(Mail_T m) {
 
 
 static void _copyMail(Mail_T n, Mail_T o) {
-        ASSERT(n);
-        ASSERT(o);
+        assert(n);
+        assert(o);
 
         n->to = Str_dup(o->to);
         if (o->from) {
@@ -244,7 +245,7 @@ static bool _send(List_T list) {
                                                 m->subject,
                                                 now,
                                                 VERSION,
-                                                (long long)Time_now(), System_randomNumber(), Run.mail_hostname ? Run.mail_hostname : Run.system->name,
+                                                (long long)Time_now(), Random_number(), Run.mail_hostname ? Run.mail_hostname : Run.system->name,
                                                 m->message) <= 0
                                    )
                                 {
@@ -292,7 +293,7 @@ static bool _hasRecipient(Mail_T list, const char *recipient) {
  * @return If failed, return Handler_Alert flag or Handler_Succeeded if succeeded
  */
 Handler_Type handle_alert(Event_T E) {
-        ASSERT(E);
+        assert(E);
 
         Handler_Type rv = Handler_Succeeded;
         Service_T s = E->source;
