@@ -59,7 +59,7 @@
 #include "system/Time.h"
 
 // libmonit
-#include "util/Convert.h"
+#include "util/Fmt.h"
 #include "exceptions/AssertException.h"
 
 
@@ -92,11 +92,11 @@ static int _getOutput(InputStream_T in, char *buf, int buflen) {
 
 
 static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, long long *timeout) {
-        ASSERT(S);
-        ASSERT(c);
-        ASSERT(msg);
+        assert(S);
+        assert(c);
+        assert(msg);
         msg[0] = 0;
-        int status = -1;
+        volatile int status = -1;
         Command_T C = NULL;
         TRY
         {
@@ -135,14 +135,13 @@ static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, long
                                 break;
                 }
                 Process_T P = Command_execute(C);
-                Command_free(&C);
                 if (P) {
                         do {
                                 Time_usleep(RETRY_INTERVAL);
                                 *timeout -= RETRY_INTERVAL;
                         } while ((status = Process_exitStatus(P)) < 0 && *timeout > 0 && ! (Run.flags & Run_Stopped));
                         if (*timeout <= 0)
-                                snprintf(msg, msglen, "Program '%s' timed out after %s", Util_commandDescription(c, (char[STRLEN]){}), Convert_time2str(_timeoutMilli, (char[11]){}));
+                                snprintf(msg, msglen, "Program '%s' timed out after %s", Util_commandDescription(c, (char[STRLEN]){}), Fmt_time2str(_timeoutMilli, (char[11]){}));
                         int n, total = 0;
                         char buf[STRLEN];
                         do {
@@ -158,7 +157,10 @@ static int _commandExecute(Service_T S, command_t c, char *msg, int msglen, long
                                 }
                         } while (n > 0 && Run.debug && total < 2048); // Limit the debug output (if the program will have endless output, such as 'yes' utility, we have to stop at some point to not spin here forever)
                         Process_free(&P); // Will kill the program if still running
+                } else {
+                        snprintf(msg, msglen, "Program %s failed: %s", c->arg[0], System_lastError());
                 }
+                Command_free(&C);
         }
         return status;
 }
@@ -193,7 +195,7 @@ static Process_Status _waitProcessStop(int pid, long long *timeout) {
 
 
 static State_Type _check(Service_T s) {
-        ASSERT(s);
+        assert(s);
         State_Type rv = State_Succeeded;
         // The check is performed in passive mode - we want to just check, nested start/stop/restart action is unwanted (alerts are allowed so the user will get feedback what's wrong)
         Monitor_Mode original = s->mode;
@@ -220,12 +222,12 @@ static State_Type _check(Service_T s) {
  * @return true if the service was started otherwise false
  */
 static bool _doStart(Service_T s) {
-        ASSERT(s);
+        assert(s);
         bool rv = true;
         StringBuffer_T sb = StringBuffer_create(64);
         for (Dependant_T d = s->dependantlist; d; d = d->next ) {
                 Service_T parent = Util_getService(d->dependant);
-                ASSERT(parent);
+                assert(parent);
                 if (parent->monitor != Monitor_Yes || parent->error) {
                         if (_doStart(parent)) {
                                 State_Type state = _check(parent);
@@ -285,7 +287,7 @@ static void _evaluateStop(Service_T s, bool succeeded, int exitStatus, char *msg
  * @return true if the service was stopped otherwise false
  */
 static bool _doStop(Service_T s, bool unmonitor) {
-        ASSERT(s);
+        assert(s);
         bool rv = true;
         if (s->stop) {
                 if (s->monitor != Monitor_Not) {
@@ -323,7 +325,7 @@ static bool _doStop(Service_T s, bool unmonitor) {
  * @param s A Service_T object
  */
 static bool _doRestart(Service_T s) {
-        ASSERT(s);
+        assert(s);
         bool rv = true;
         if (s->restart) {
                 Log_info("'%s' restart: '%s'\n", s->name, Util_commandDescription(s->restart, (char[STRLEN]){}));
@@ -351,10 +353,10 @@ static bool _doRestart(Service_T s) {
  * @param s A Service_T object
  */
 static void _doMonitor(Service_T s) {
-        ASSERT(s);
+        assert(s);
         for (Dependant_T d = s->dependantlist; d; d = d->next ) {
                 Service_T parent = Util_getService(d->dependant);
-                ASSERT(parent);
+                assert(parent);
                 _doMonitor(parent);
         }
         Util_monitorSet(s);
@@ -366,7 +368,7 @@ static void _doMonitor(Service_T s) {
  * @param s A Service_T object
  */
 static void _doUnmonitor(Service_T s) {
-        ASSERT(s);
+        assert(s);
         Util_monitorUnset(s);
 }
 
@@ -379,9 +381,9 @@ static void _doUnmonitor(Service_T s) {
  * @return true if all depending services were started/stopped otherwise false
  */
 static bool _doDepend(Service_T s, Action_Type action, bool unmonitor) {
-        ASSERT(s);
+        assert(s);
         bool rv = true;
-        for (Service_T child = servicelist; child; child = child->next) {
+        for (Service_T child = Service_List; child; child = child->next) {
                 for (Dependant_T d = child->dependantlist; d; d = d->next) {
                         if (IS(d->dependant, s->name)) {
                                 if (action == Action_Start) {
@@ -428,15 +430,15 @@ static bool _doDepend(Service_T s, Action_Type action, bool unmonitor) {
  * @return number of errors
  */
 bool control_service_string(List_T services, const char *action) {
-        ASSERT(services);
-        ASSERT(action);
+        assert(services);
+        assert(action);
         Action_Type a = Util_getAction(action);
         if (a == Action_Ignored) {
                 Log_error("invalid action %s\n", action);
                 return 1;
         }
         int errors = 0;
-        for (list_t s = services->head; s; s = s->next)
+        for (_list_t s = services->head; s; s = s->next)
                 if (control_service(s->e, a) == false)
                         errors++;
         return errors;
@@ -452,7 +454,7 @@ bool control_service_string(List_T services, const char *action) {
 bool control_service(const char *S, Action_Type A) {
         Service_T s = NULL;
         bool rv = true;
-        ASSERT(S);
+        assert(S);
         if (! (s = Util_getService(S))) {
                 Log_error("Service '%s' -- doesn't exist\n", S);
                 return false;
