@@ -57,11 +57,13 @@ static void _gc_action(Action_T *);
 static void _gc_eventaction(EventAction_T *);
 static void _gcpdl(Dependant_T *);
 static void _gcso(Size_T *);
+static void _gcnlink(NLink_T *);
 static void _gclinkstatus(LinkStatus_T *);
 static void _gclinkspeed(LinkSpeed_T *);
 static void _gclinksaturation(LinkSaturation_T *);
 static void _gcbandwidth(Bandwidth_T *);
 static void _gcmatch(Match_T *);
+static void _gcoutputchange(OutputChange_T *);
 static void _gcchecksum(Checksum_T *);
 static void _gcperm(Perm_T *);
 static void _gcstatus(Status_T *);
@@ -92,14 +94,14 @@ static void _gcfiledescriptors(Filedescriptors_T *);
 /* ------------------------------------------------------------------ Public */
 
 
-void gc() {
+void gc(void) {
         Engine_destroyAllow();
         if (Run.flags & Run_ProcessEngineEnabled)
                 ProcessTree_delete();
-        if (servicelist)
-                _gc_service_list(&servicelist);
-        if (servicegrouplist)
-                _gc_servicegroup(&servicegrouplist);
+        if (Service_List)
+                _gc_service_list(&Service_List);
+        if (Service_Group_List)
+                _gc_servicegroup(&Service_Group_List);
         if (Run.httpd.credentials)
                 _gcath(&Run.httpd.credentials);
         if (Run.maillist)
@@ -127,7 +129,7 @@ void gc() {
 
 
 void gc_mail_list(Mail_T *m) {
-        ASSERT(m);
+        assert(m);
         if ((*m)->next)
                 gc_mail_list(&(*m)->next);
         if ((*m)->from)
@@ -142,7 +144,7 @@ void gc_mail_list(Mail_T *m) {
 
 
 void gccmd(command_t *c) {
-        ASSERT(c && *c);
+        assert(c && *c);
         for (int i = 0; (*c)->arg[i]; i++)
                 FREE((*c)->arg[i]);
         FREE(*c);
@@ -150,7 +152,7 @@ void gccmd(command_t *c) {
 
 
 void gc_event(Event_T *e) {
-        ASSERT(e && *e);
+        assert(e && *e);
         if ((*e)->next)
                 gc_event(&(*e)->next);
         (*e)->action = NULL;
@@ -159,6 +161,17 @@ void gc_event(Event_T *e) {
 }
 
 
+static void _gc_hostgroups(List_T *hostgroups) {
+        if (*hostgroups) {
+                char *hostgroup = NULL;
+                while ((hostgroup = List_pop(*hostgroups))) {
+                        FREE(hostgroup);
+                }
+                List_free(hostgroups);
+        }
+}
+
+
 /* ----------------------------------------------------------------- Private */
 
 
@@ -175,7 +188,7 @@ static void _gcssloptions(SslOptions_T o) {
 
 
 static void _gc_service_list(Service_T *s) {
-        ASSERT(s&&*s);
+        assert(s&&*s);
         if ((*s)->next)
                 _gc_service_list(&(*s)->next);
         _gc_service(&(*s));
@@ -183,7 +196,7 @@ static void _gc_service_list(Service_T *s) {
 
 
 static void _gc_service(Service_T *s) {
-        ASSERT(s&&*s);
+        assert(s&&*s);
         if ((*s)->program) {
                 if ((*s)->program->P)
                         Process_free(&(*s)->program->P);
@@ -215,6 +228,8 @@ static void _gc_service(Service_T *s) {
                 _gcparl(&(*s)->actionratelist);
         if ((*s)->sizelist)
                 _gcso(&(*s)->sizelist);
+        if ((*s)->nlinklist)
+                _gcnlink(&(*s)->nlinklist);
         if ((*s)->linkstatuslist)
                 _gclinkstatus(&(*s)->linkstatuslist);
         if ((*s)->linkspeedlist)
@@ -233,6 +248,8 @@ static void _gc_service(Service_T *s) {
                 _gcmatch(&(*s)->matchlist);
         if ((*s)->matchignorelist)
                 _gcmatch(&(*s)->matchignorelist);
+        if ((*s)->outputchangelist)
+                _gcoutputchange(&(*s)->outputchangelist);
         if ((*s)->checksum)
                 _gcchecksum(&(*s)->checksum);
         if ((*s)->perm)
@@ -316,7 +333,7 @@ static void _gc_service(Service_T *s) {
 
 
 static void _gc_servicegroup(ServiceGroup_T *sg) {
-        ASSERT(sg && *sg);
+        assert(sg && *sg);
         if ((*sg)->next)
                 _gc_servicegroup(&(*sg)->next);
         List_free(&(*sg)->members);
@@ -326,7 +343,7 @@ static void _gc_servicegroup(ServiceGroup_T *sg) {
 
 
 static void _gc_request(Request_T *r) {
-        ASSERT(r);
+        assert(r);
         if ((*r)->url)
                 _gc_url(&(*r)->url);
         if ((*r)->regex)
@@ -337,7 +354,7 @@ static void _gc_request(Request_T *r) {
 
 
 static void _gc_url(URL_T *url) {
-        ASSERT(url);
+        assert(url);
         FREE((*url)->url);
         FREE((*url)->protocol);
         FREE((*url)->user);
@@ -363,7 +380,7 @@ static void _gc_mail_server(MailServer_T *s) {
 
 
 static void _gc_action(Action_T *a) {
-        ASSERT(a&&*a);
+        assert(a&&*a);
         if ((*a)->exec)
                 gccmd(&(*a)->exec);
         FREE(*a);
@@ -371,7 +388,7 @@ static void _gc_action(Action_T *a) {
 
 
 static void _gc_eventaction(EventAction_T *e) {
-        ASSERT(e&&*e);
+        assert(e&&*e);
         _gc_action(&(*e)->failed);
         _gc_action(&(*e)->succeeded);
         FREE(*e);
@@ -379,7 +396,7 @@ static void _gc_eventaction(EventAction_T *e) {
 
 
 static void _gcportlist(Port_T *p) {
-        ASSERT(p&&*p);
+        assert(p&&*p);
         if ((*p)->next)
                 _gcportlist(&(*p)->next);
         if ((*p)->action)
@@ -440,7 +457,7 @@ static void _gcportlist(Port_T *p) {
 
 
 static void _gcfilesystem(FileSystem_T *d) {
-        ASSERT(d&&*d);
+        assert(d&&*d);
         if ((*d)->next)
                 _gcfilesystem(&(*d)->next);
         if ((*d)->action)
@@ -450,7 +467,7 @@ static void _gcfilesystem(FileSystem_T *d) {
 
 
 static void _gcicmp(Icmp_T *i) {
-        ASSERT(i&&*i);
+        assert(i&&*i);
         if ((*i)->next)
                 _gcicmp(&(*i)->next);
         FREE((*i)->outgoing.ip);
@@ -461,7 +478,7 @@ static void _gcicmp(Icmp_T *i) {
 
 
 static void _gcpql(Resource_T *q) {
-        ASSERT(q);
+        assert(q);
         if ((*q)->next)
                 _gcpql(&(*q)->next);
         if ((*q)->action)
@@ -471,7 +488,7 @@ static void _gcpql(Resource_T *q) {
 
 
 static void _gcptl(Timestamp_T *p) {
-        ASSERT(p);
+        assert(p);
         if ((*p)->next)
                 _gcptl(&(*p)->next);
         if ((*p)->action)
@@ -481,7 +498,7 @@ static void _gcptl(Timestamp_T *p) {
 
 
 static void _gcuptimelist(Uptime_T *u) {
-        ASSERT(u);
+        assert(u);
         if ((*u)->next)
                 _gcuptimelist(&(*u)->next);
         if ((*u)->action)
@@ -491,7 +508,7 @@ static void _gcuptimelist(Uptime_T *u) {
 
 
 static void _gcparl(ActionRate_T *ar) {
-        ASSERT(ar);
+        assert(ar);
         if ((*ar)->next)
                 _gcparl(&(*ar)->next);
         if ((*ar)->action)
@@ -501,7 +518,7 @@ static void _gcparl(ActionRate_T *ar) {
 
 
 static void _gcso(Size_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->next)
                 _gcso(&(*s)->next);
         if ((*s)->action)
@@ -509,8 +526,17 @@ static void _gcso(Size_T *s) {
         FREE(*s);
 }
 
+static void _gcnlink(NLink_T *s) {
+        assert(s);
+        if ((*s)->next)
+                _gcnlink(&(*s)->next);
+        if ((*s)->action)
+                _gc_eventaction(&(*s)->action);
+        FREE(*s);
+}
+
 static void _gclinkstatus(LinkStatus_T *l) {
-        ASSERT(l);
+        assert(l);
         if ((*l)->next)
                 _gclinkstatus(&(*l)->next);
         if ((*l)->action)
@@ -520,7 +546,7 @@ static void _gclinkstatus(LinkStatus_T *l) {
 
 
 static void _gclinkspeed(LinkSpeed_T *l) {
-        ASSERT(l);
+        assert(l);
         if ((*l)->next)
                 _gclinkspeed(&(*l)->next);
         if ((*l)->action)
@@ -530,7 +556,7 @@ static void _gclinkspeed(LinkSpeed_T *l) {
 
 
 static void _gclinksaturation(LinkSaturation_T *l) {
-        ASSERT(l);
+        assert(l);
         if ((*l)->next)
                 _gclinksaturation(&(*l)->next);
         if ((*l)->action)
@@ -540,7 +566,7 @@ static void _gclinksaturation(LinkSaturation_T *l) {
 
 
 static void _gcbandwidth(Bandwidth_T *b) {
-        ASSERT(b);
+        assert(b);
         if ((*b)->next)
                 _gcbandwidth(&(*b)->next);
         if ((*b)->action)
@@ -548,8 +574,9 @@ static void _gcbandwidth(Bandwidth_T *b) {
         FREE(*b);
 }
 
+
 static void _gcmatch(Match_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->next)
                 _gcmatch(&(*s)->next);
         if ((*s)->action)
@@ -564,8 +591,19 @@ static void _gcmatch(Match_T *s) {
 }
 
 
+static void _gcoutputchange(OutputChange_T *s) {
+        assert(s);
+        if ((*s)->next)
+                _gcoutputchange(&(*s)->next);
+        if ((*s)->action)
+                _gc_eventaction(&(*s)->action);
+        FREE((*s)->previous);
+        FREE(*s);
+}
+
+
 static void _gcchecksum(Checksum_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->action)
                 _gc_eventaction(&(*s)->action);
         FREE(*s);
@@ -573,7 +611,7 @@ static void _gcchecksum(Checksum_T *s) {
 
 
 static void _gcperm(Perm_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->action)
                 _gc_eventaction(&(*s)->action);
         FREE(*s);
@@ -581,7 +619,7 @@ static void _gcperm(Perm_T *s) {
 
 
 static void _gcstatus(Status_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->next)
                 _gcstatus(&(*s)->next);
         if ((*s)->action)
@@ -591,7 +629,7 @@ static void _gcstatus(Status_T *s) {
 
 
 static void _gcuid(Uid_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->action)
                 _gc_eventaction(&(*s)->action);
         FREE(*s);
@@ -599,7 +637,7 @@ static void _gcuid(Uid_T *s) {
 
 
 static void _gcgid(Gid_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->action)
                 _gc_eventaction(&(*s)->action);
         FREE(*s);
@@ -607,7 +645,7 @@ static void _gcgid(Gid_T *s) {
 
 
 static void _gcpid(Pid_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->next)
                 _gcpid(&(*s)->next);
         if ((*s)->action)
@@ -617,7 +655,7 @@ static void _gcpid(Pid_T *s) {
 
 
 static void _gcppid(Pid_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->next)
                 _gcppid(&(*s)->next);
         if ((*s)->action)
@@ -627,7 +665,7 @@ static void _gcppid(Pid_T *s) {
 
 
 static void _gcfsflag(FsFlag_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->next)
                 _gcfsflag(&(*s)->next);
         if ((*s)->action)
@@ -637,7 +675,7 @@ static void _gcfsflag(FsFlag_T *s) {
 
 
 static void _gcnonexist(NonExist_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->next)
                 _gcnonexist(&(*s)->next);
         if ((*s)->action)
@@ -647,7 +685,7 @@ static void _gcnonexist(NonExist_T *s) {
 
 
 static void _gcexist(Exist_T *s) {
-        ASSERT(s);
+        assert(s);
         if ((*s)->next)
                 _gcexist(&(*s)->next);
         if ((*s)->action)
@@ -657,7 +695,7 @@ static void _gcexist(Exist_T *s) {
 
 
 static void _gcpdl(Dependant_T *d) {
-        ASSERT(d);
+        assert(d);
         if ((*d)->next)
                 _gcpdl(&(*d)->next);
         StringBuffer_free(&((*d)->dependant_htmlescaped));
@@ -668,7 +706,7 @@ static void _gcpdl(Dependant_T *d) {
 
 
 static void _gcgeneric(Generic_T *g) {
-        ASSERT(g);
+        assert(g);
         if ((*g)->next)
                 _gcgeneric(&(*g)->next);
         FREE((*g)->send);
@@ -681,7 +719,7 @@ static void _gcgeneric(Generic_T *g) {
 
 
 static void _gcath(Auth_T *c) {
-        ASSERT(c);
+        assert(c);
         if ((*c)->next)
                 _gcath(&(*c)->next);
         FREE((*c)->uname);
@@ -691,18 +729,21 @@ static void _gcath(Auth_T *c) {
 }
 
 
-static void _gc_mmonit(Mmonit_T *recv) {
-        ASSERT(recv);
-        if ((*recv)->next)
-                _gc_mmonit(&(*recv)->next);
-        _gc_url(&(*recv)->url);
-        _gcssloptions(&((*recv)->ssl));
-        FREE(*recv);
+static void _gc_mmonit(Mmonit_T *mmonit) {
+        assert(mmonit);
+
+        if ((*mmonit)->next)
+                _gc_mmonit(&(*mmonit)->next);
+
+        _gc_url(&(*mmonit)->url);
+        _gcssloptions(&((*mmonit)->ssl));
+        _gc_hostgroups(&((*mmonit)->hostgroups));
+        FREE(*mmonit);
 }
 
 
 static void _gcsecattr(SecurityAttribute_T *s) {
-        ASSERT(s && *s);
+        assert(s && *s);
         if ((*s)->next)
                 _gcsecattr(&(*s)->next);
         if ((*s)->action)
@@ -713,7 +754,7 @@ static void _gcsecattr(SecurityAttribute_T *s) {
 
 
 static void _gcfiledescriptors(Filedescriptors_T *o) {
-        ASSERT(o && *o);
+        assert(o && *o);
         if ((*o)->next)
                 _gcfiledescriptors(&(*o)->next);
         if ((*o)->action)
